@@ -15,6 +15,7 @@
 # under the License.
 import logging
 
+from oslo_utils import encodeutils
 import pymysql
 
 import tooz
@@ -62,9 +63,10 @@ class MySQLLock(locking.Lock):
                         self.acquired = True
                         return True
             except pymysql.MySQLError as e:
-                coordination.raise_with_cause(coordination.ToozError,
-                                              utils.exception_message(e),
-                                              cause=e)
+                coordination.raise_with_cause(
+                    coordination.ToozError,
+                    encodeutils.exception_to_unicode(e),
+                    cause=e)
 
             if blocking:
                 raise _retry.Retry
@@ -83,12 +85,12 @@ class MySQLLock(locking.Lock):
                 return True
         except pymysql.MySQLError as e:
             coordination.raise_with_cause(coordination.ToozError,
-                                          utils.exception_message(e),
+                                          encodeutils.exception_to_unicode(e),
                                           cause=e)
 
     def __del__(self):
         if self.acquired:
-            LOG.warn("unreleased lock %s garbage collected" % self.name)
+            LOG.warning("unreleased lock %s garbage collected", self.name)
 
 
 class MySQLDriver(coordination.CoordinationDriver):
@@ -102,6 +104,16 @@ class MySQLDriver(coordination.CoordinationDriver):
     .. _MySQL: http://dev.mysql.com/
     """
 
+    CHARACTERISTICS = (
+        coordination.Characteristics.DISTRIBUTED_ACROSS_THREADS,
+        coordination.Characteristics.DISTRIBUTED_ACROSS_PROCESSES,
+        coordination.Characteristics.DISTRIBUTED_ACROSS_HOSTS,
+    )
+    """
+    Tuple of :py:class:`~tooz.coordination.Characteristics` introspectable
+    enum member(s) that can be used to interogate how this driver works.
+    """
+
     def __init__(self, member_id, parsed_url, options):
         """Initialize the MySQL driver."""
         super(MySQLDriver, self).__init__()
@@ -166,5 +178,5 @@ class MySQLDriver(coordination.CoordinationDriver):
                                        database=dbname)
         except (pymysql.err.OperationalError, pymysql.err.InternalError) as e:
             coordination.raise_with_cause(coordination.ToozConnectionError,
-                                          utils.exception_message(e),
+                                          encodeutils.exception_to_unicode(e),
                                           cause=e)
